#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2013-17 Richard Hull and contributors
# See LICENSE.rst for details.

"""
Tests for the :py:class:`luma.lcd.device.st7567` device.
"""

import pytest

from luma.lcd.device import st7567
from luma.core.render import canvas

from baseline_data import get_reference_data, primitives
from helpers import serial, setup_function, assert_invalid_dimensions  # noqa: F401
from unittest.mock import Mock, call


def test_init_128x64():
    st7567(serial, gpio=Mock())
    serial.command.assert_has_calls([
        call(0xA3),
        call(0xA1),
        call(0xC0),
        call(0xA6),
        call(0x40),
        call(0x2F),
        call(0x22),
        call(0xAF),
        call(0x81, 57)
    ])

    # Next 1024 are all data: zeros to clear the RAM
    # (1024 = 128 * 64 / 8)
    serial.data.assert_has_calls([call([0] * 128)] * 8)


def test_contrast():
    device = st7567(serial, gpio=Mock())
    serial.reset_mock()
    with pytest.raises(AssertionError):
        device.contrast(300)


def test_display():
    device = st7567(serial, gpio=Mock())
    serial.reset_mock()

    recordings = []

    def data(data):
        recordings.append({'data': data})

    def command(*cmd):
        recordings.append({'command': list(cmd)})

    serial.command.side_effect = command
    serial.data.side_effect = data

    # Use the same drawing primitives as the demo
    with canvas(device) as draw:
        primitives(device, draw)

    assert serial.data.called
    assert serial.command.called

    # To regenerate test data, uncomment the following (remember not to commit though)
    # ================================================================================
    # from baseline_data import save_reference_data
    # save_reference_data("demo_st7567", recordings)

    assert recordings == get_reference_data('demo_st7567')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2013-19 Richard Hull and contributors
# See LICENSE.rst for details.

"""
Tests for the :py:class:`luma.lcd.device.ili9341` device.
"""

import pytest

from luma.lcd.device import ili9341
from luma.core.render import canvas
from luma.core.framebuffer import full_frame

from baseline_data import get_reference_data, primitives
from helpers import serial, setup_function, assert_invalid_dimensions  # noqa: F401
from unittest.mock import Mock


def test_init_320x240():
    recordings = []

    def data(data):
        recordings.append({'data': data})

    def command(*cmd):
        recordings.append({'command': list(cmd)})

    serial.command.side_effect = command
    serial.data.side_effect = data

    ili9341(serial, gpio=Mock(), framebuffer=full_frame())

    assert serial.data.called
    assert serial.command.called

    assert recordings == [
        {'command': [0xef]}, {'data': [0x03, 0x80, 0x02]},
        {'command': [0xcf]}, {'data': [0x00, 0xc1, 0x30]},
        {'command': [0xed]}, {'data': [0x64, 0x03, 0x12, 0x81]},
        {'command': [0xe8]}, {'data': [0x85, 0x00, 0x78]},
        {'command': [0xcb]}, {'data': [0x39, 0x2c, 0x00, 0x34, 0x02]},
        {'command': [0xf7]}, {'data': [0x20]},
        {'command': [0xea]}, {'data': [0x00, 0x00]},
        {'command': [0xc0]}, {'data': [0x23]},
        {'command': [0xc1]}, {'data': [0x10]},
        {'command': [0xc5]}, {'data': [0x3e, 0x28]},
        {'command': [0xc7]}, {'data': [0x86]},
        {'command': [0x36]}, {'data': [0x28]},
        {'command': [0x3a]}, {'data': [0x46]},
        {'command': [0xb1]}, {'data': [0x00, 0x18]},
        {'command': [0xb6]}, {'data': [0x08, 0x82, 0x27]},
        {'command': [0xf2]}, {'data': [0x00]},
        {'command': [0x26]}, {'data': [0x01]},
        {'command': [0xe0]}, {'data': [0x0f, 0x31, 0x2b, 0x0c, 0x0e, 0x08, 0x4e, 0xf1, 0x37, 0x07, 0x10, 0x03, 0x0e, 0x09, 0x00]},
        {'command': [0xe1]}, {'data': [0x00, 0x0e, 0x14, 0x03, 0x11, 0x07, 0x31, 0xc1, 0x48, 0x08, 0x0f, 0x0c, 0x31, 0x36, 0x0f]},
        {'command': [0x11]},
        {'command': [0x2a]}, {'data': [0x00, 0x00, 0x01, 0x3f]},
        {'command': [0x2b]}, {'data': [0x00, 0x00, 0x00, 0xef]},
        {'command': [0x2c]},
        {'data': bytearray([0x00] * (320 * 240 * 3))},
        {'command': [0x29]},
    ]


def test_init_240x240():
    recordings = []

    def data(data):
        recordings.append({'data': data})

    def command(*cmd):
        recordings.append({'command': list(cmd)})

    serial.command.side_effect = command
    serial.data.side_effect = data

    ili9341(serial, gpio=Mock(), width=240, height=240, framebuffer=full_frame())

    assert serial.data.called
    assert serial.command.called

    assert recordings == [
        {'command': [0xef]}, {'data': [0x03, 0x80, 0x02]},
        {'command': [0xcf]}, {'data': [0x00, 0xc1, 0x30]},
        {'command': [0xed]}, {'data': [0x64, 0x03, 0x12, 0x81]},
        {'command': [0xe8]}, {'data': [0x85, 0x00, 0x78]},
        {'command': [0xcb]}, {'data': [0x39, 0x2c, 0x00, 0x34, 0x02]},
        {'command': [0xf7]}, {'data': [0x20]},
        {'command': [0xea]}, {'data': [0x00, 0x00]},
        {'command': [0xc0]}, {'data': [0x23]},
        {'command': [0xc1]}, {'data': [0x10]},
        {'command': [0xc5]}, {'data': [0x3e, 0x28]},
        {'command': [0xc7]}, {'data': [0x86]},
        {'command': [0x36]}, {'data': [0x28]},
        {'command': [0x3a]}, {'data': [0x46]},
        {'command': [0xb1]}, {'data': [0x00, 0x18]},
        {'command': [0xb6]}, {'data': [0x08, 0x82, 0x27]},
        {'command': [0xf2]}, {'data': [0x00]},
        {'command': [0x26]}, {'data': [0x01]},
        {'command': [0xe0]}, {'data': [0x0f, 0x31, 0x2b, 0x0c, 0x0e, 0x08, 0x4e, 0xf1, 0x37, 0x07, 0x10, 0x03, 0x0e, 0x09, 0x00]},
        {'command': [0xe1]}, {'data': [0x00, 0x0e, 0x14, 0x03, 0x11, 0x07, 0x31, 0xc1, 0x48, 0x08, 0x0f, 0x0c, 0x31, 0x36, 0x0f]},
        {'command': [0x11]},
        {'command': [0x2A]}, {'data': [0x00, 0x00, 0x00, 0xef]},
        {'command': [0x2B]}, {'data': [0x00, 0x00, 0x00, 0xef]},
        {'command': [0x2C]},
        {'data': bytearray([0x00] * (240 * 240 * 3))},
        {'command': [0x29]},
    ]


def test_init_320x180():
    recordings = []

    def data(data):
        recordings.append({'data': data})

    def command(*cmd):
        recordings.append({'command': list(cmd)})

    serial.command.side_effect = command
    serial.data.side_effect = data

    ili9341(serial, gpio=Mock(), width=320, height=180, framebuffer=full_frame())

    assert serial.data.called
    assert serial.command.called

    assert recordings == [
        {'command': [0xef]}, {'data': [0x03, 0x80, 0x02]},
        {'command': [0xcf]}, {'data': [0x00, 0xc1, 0x30]},
        {'command': [0xed]}, {'data': [0x64, 0x03, 0x12, 0x81]},
        {'command': [0xe8]}, {'data': [0x85, 0x00, 0x78]},
        {'command': [0xcb]}, {'data': [0x39, 0x2c, 0x00, 0x34, 0x02]},
        {'command': [0xf7]}, {'data': [0x20]},
        {'command': [0xea]}, {'data': [0x00, 0x00]},
        {'command': [0xc0]}, {'data': [0x23]},
        {'command': [0xc1]}, {'data': [0x10]},
        {'command': [0xc5]}, {'data': [0x3e, 0x28]},
        {'command': [0xc7]}, {'data': [0x86]},
        {'command': [0x36]}, {'data': [0x28]},
        {'command': [0x3a]}, {'data': [0x46]},
        {'command': [0xb1]}, {'data': [0x00, 0x18]},
        {'command': [0xb6]}, {'data': [0x08, 0x82, 0x27]},
        {'command': [0xf2]}, {'data': [0x00]},
        {'command': [0x26]}, {'data': [0x01]},
        {'command': [0xe0]}, {'data': [0x0f, 0x31, 0x2b, 0x0c, 0x0e, 0x08, 0x4e, 0xf1, 0x37, 0x07, 0x10, 0x03, 0x0e, 0x09, 0x00]},
        {'command': [0xe1]}, {'data': [0x00, 0x0e, 0x14, 0x03, 0x11, 0x07, 0x31, 0xc1, 0x48, 0x08, 0x0f, 0x0c, 0x31, 0x36, 0x0f]},
        {'command': [0x11]},
        {'command': [0x2A]}, {'data': [0x00, 0x00, 0x01, 0x3f]},
        {'command': [0x2B]}, {'data': [0x00, 0x00, 0x00, 0xb3]},
        {'command': [0x2C]},
        {'data': bytearray([0x00] * (320 * 180 * 3))},
        {'command': [0x29]},
    ]


def test_init_invalid_dimensions():
    """
    ILI9341 LCD with an invalid resolution raises a
    :py:class:`luma.core.error.DeviceDisplayModeError`.
    """
    assert_invalid_dimensions(ili9341, serial, 128, 77)


def test_offsets():
    recordings = []

    def data(data):
        recordings.append({'data': data})

    def command(*cmd):
        recordings.append({'command': list(cmd)})

    serial.command.side_effect = command
    serial.data.side_effect = data

    ili9341(serial, gpio=Mock(), width=240, height=240, h_offset=2, v_offset=1, framebuffer=full_frame())

    assert serial.data.called
    assert serial.command.called

    assert recordings == [
        {'command': [0xef]}, {'data': [0x03, 0x80, 0x02]},
        {'command': [0xcf]}, {'data': [0x00, 0xc1, 0x30]},
        {'command': [0xed]}, {'data': [0x64, 0x03, 0x12, 0x81]},
        {'command': [0xe8]}, {'data': [0x85, 0x00, 0x78]},
        {'command': [0xcb]}, {'data': [0x39, 0x2c, 0x00, 0x34, 0x02]},
        {'command': [0xf7]}, {'data': [0x20]},
        {'command': [0xea]}, {'data': [0x00, 0x00]},
        {'command': [0xc0]}, {'data': [0x23]},
        {'command': [0xc1]}, {'data': [0x10]},
        {'command': [0xc5]}, {'data': [0x3e, 0x28]},
        {'command': [0xc7]}, {'data': [0x86]},
        {'command': [0x36]}, {'data': [0x28]},
        {'command': [0x3a]}, {'data': [0x46]},
        {'command': [0xb1]}, {'data': [0x00, 0x18]},
        {'command': [0xb6]}, {'data': [0x08, 0x82, 0x27]},
        {'command': [0xf2]}, {'data': [0x00]},
        {'command': [0x26]}, {'data': [0x01]},
        {'command': [0xe0]}, {'data': [0x0f, 0x31, 0x2b, 0x0c, 0x0e, 0x08, 0x4e, 0xf1, 0x37, 0x07, 0x10, 0x03, 0x0e, 0x09, 0x00]},
        {'command': [0xe1]}, {'data': [0x00, 0x0e, 0x14, 0x03, 0x11, 0x07, 0x31, 0xc1, 0x48, 0x08, 0x0f, 0x0c, 0x31, 0x36, 0x0f]},
        {'command': [0x11]},
        {'command': [0x2A]}, {'data': [0x00, 0x02, 0x00, 0xef + 0x02]},
        {'command': [0x2B]}, {'data': [0x00, 0x01, 0x00, 0xef + 0x01]},
        {'command': [0x2C]},
        {'data': bytearray([0x00] * (240 * 240 * 3))},
        {'command': [0x29]},
    ]


def test_contrast():
    device = ili9341(serial, gpio=Mock())
    serial.reset_mock()
    with pytest.raises(AssertionError):
        device.contrast(300)


def test_hide():
    device = ili9341(serial, gpio=Mock())
    serial.reset_mock()
    device.hide()
    serial.command.assert_called_once_with(40)


def test_show():
    device = ili9341(serial, gpio=Mock())
    serial.reset_mock()
    device.show()
    serial.command.assert_called_once_with(41)


def test_display_full_frame():
    device = ili9341(serial, gpio=Mock(), framebuffer=full_frame())
    serial.reset_mock()

    recordings = []

    def data(data):
        recordings.append({'data': list(data)})

    def command(*cmd):
        recordings.append({'command': list(cmd)})

    serial.command.side_effect = command
    serial.data.side_effect = data

    # Use the same drawing primitives as the demo
    with canvas(device) as draw:
        primitives(device, draw)

    assert serial.data.called
    assert serial.command.called

    # To regenerate test data, uncomment the following (remember not to commit though)
    # ================================================================================
    # from baseline_data import save_reference_data
    # save_reference_data("demo_ili9341", recordings)

    assert recordings == get_reference_data('demo_ili9341')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2020 Richard Hull and contributors
# See LICENSE.rst for details.

"""
Tests for the :py:class:`luma.lcd.device.ili9486` device.
"""

import pytest

from luma.lcd.device import ili9486
from luma.core.render import canvas
from luma.core.framebuffer import full_frame

from baseline_data import get_reference_data, primitives
from helpers import serial, setup_function, assert_invalid_dimensions  # noqa: F401
from unittest.mock import Mock


def test_init_320x480():
    recordings = []

    def data(data):
        recordings.append({'data': data})

    def command(*cmd):
        recordings.append({'command': list(cmd)})

    serial.command.side_effect = command
    serial.data.side_effect = data

    ili9486(serial, gpio=Mock(), framebuffer=full_frame())

    assert serial.data.called
    assert serial.command.called

    # This set of expected results include the padding bytes that
    # appear necessary with Waveshare's ili9486 implementation.
    assert recordings == [
        {'command': [0xb0]}, {'data': [0x00, 0x00]},
        {'command': [0x11]},
        {'command': [0x3a]}, {'data': [0x00, 0x66]},
        {'command': [0x21]},
        {'command': [0xc0]}, {'data': [0x00, 0x09, 0x00, 0x09]},
        {'command': [0xc1]}, {'data': [0x00, 0x41, 0x00, 0x00]},
        {'command': [0xc2]}, {'data': [0x00, 0x33]},
        {'command': [0xc5]}, {'data': [0x00, 0x00, 0x00, 0x36]},
        {'command': [0x36]}, {'data': [0x00, 0x08]},
        {'command': [0xb6]}, {'data': [0x00, 0x00, 0x00, 0x42, 0x00, 0x3b]},
        {'command': [0x13]},
        {'command': [0x34]},
        {'command': [0x38]},
        {'command': [0x11]},
        {'command': [0x2a]}, {'data': [0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x3f]},
        {'command': [0x2b]}, {'data': [0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0xdf]},
        {'command': [0x2c]},
        {'data': bytearray([0x00] * (320 * 480 * 3))},
        {'command': [0x29]},
    ]


def test_init_invalid_dimensions():
    """
    ILI9486 LCD with an invalid resolution raises a
    :py:class:`luma.core.error.DeviceDisplayModeError`.
    """
    assert_invalid_dimensions(ili9486, serial, 128, 77)


def test_offsets():
    recordings = []

    def data(data):
        recordings.append({'data': data})

    def command(*cmd):
        recordings.append({'command': list(cmd)})

    serial.command.side_effect = command
    serial.data.side_effect = data

    ili9486(serial, gpio=Mock(), width=320, height=480, h_offset=2, v_offset=1, framebuffer=full_frame())

    assert serial.data.called
    assert serial.command.called

    assert recordings == [
        {'command': [0xb0]}, {'data': [0x00, 0x00]},
        {'command': [0x11]},
        {'command': [0x3a]}, {'data': [0x00, 0x66]},
        {'command': [0x21]},
        {'command': [0xc0]}, {'data': [0x00, 0x09, 0x00, 0x09]},
        {'command': [0xc1]}, {'data': [0x00, 0x41, 0x00, 0x00]},
        {'command': [0xc2]}, {'data': [0x00, 0x33]},
        {'command': [0xc5]}, {'data': [0x00, 0x00, 0x00, 0x36]},
        {'command': [0x36]}, {'data': [0x00, 0x08]},
        {'command': [0xb6]}, {'data': [0x00, 0x00, 0x00, 0x42, 0x00, 0x3b]},
        {'command': [0x13]},
        {'command': [0x34]},
        {'command': [0x38]},
        {'command': [0x11]},
        {'command': [0x2A]}, {'data': [0x00, 0x00, 0x00, 0x02, 0x00, 0x01, 0x00, 0x3f + 0x02]},
        {'command': [0x2B]}, {'data': [0x00, 0x00, 0x00, 0x01, 0x00, 0x01, 0x00, 0xdf + 0x01]},
        {'command': [0x2C]},
        {'data': bytearray([0x00] * (320 * 480 * 3))},
        {'command': [0x29]},
    ]


def test_contrast():
    device = ili9486(serial, gpio=Mock())
    serial.reset_mock()
    with pytest.raises(AssertionError):
        device.contrast(300)


def test_hide():
    device = ili9486(serial, gpio=Mock())
    serial.reset_mock()
    device.hide()
    serial.command.assert_called_once_with(40)


def test_show():
    device = ili9486(serial, gpio=Mock())
    serial.reset_mock()
    device.show()
    serial.command.assert_called_once_with(41)


def test_display_full_frame():
    device = ili9486(serial, gpio=Mock(), framebuffer=full_frame())
    serial.reset_mock()

    recordings = []

    def data(data):
        recordings.append({'data': list(data)})

    def command(*cmd):
        recordings.append({'command': list(cmd)})

    serial.command.side_effect = command
    serial.data.side_effect = data

    # Use the same drawing primitives as the demo
    with canvas(device) as draw:
        primitives(device, draw)

    assert serial.data.called
    assert serial.command.called

    # To regenerate test data, uncomment the following (remember not to commit though)
    # ================================================================================
    # from baseline_data import save_reference_data
    # save_reference_data("demo_ili9486", recordings)

    assert recordings == get_reference_data('demo_ili9486')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2013-2020 Richard Hull and contributors
# See LICENSE.rst for details.

import json
from pathlib import Path


__all__ = ['primitives', 'get_reference_data']


def primitives(device, draw):
    padding = 2
    shape_width = 20
    top = padding
    bottom = device.height - padding - 1
    draw.rectangle(device.bounding_box, outline="white", fill="black")
    x = padding
    draw.ellipse((x, top, x + shape_width, bottom), outline="red", fill="black")
    x += shape_width + padding
    draw.rectangle((x, top, x + shape_width, bottom), outline="blue", fill="black")
    x += shape_width + padding
    draw.polygon([(x, bottom), (x + shape_width / 2, top), (x + shape_width, bottom)], outline="green", fill="black")
    x += shape_width + padding
    draw.line((x, bottom, x + shape_width, top), fill="yellow")
    draw.line((x, top, x + shape_width, bottom), fill="yellow")
    x += shape_width + padding
    draw.text((x, top), 'Hello', fill="cyan")
    draw.text((x, top + 20), 'World!', fill="purple")


def get_reference_data(fname):
    """
    Load JSON reference data.

    :param fname: Filename without extension.
    :type fname: str
    """
    base_dir = Path(__file__).resolve().parent
    fpath = base_dir.joinpath('reference', 'data', fname + '.json')
    with fpath.open() as f:
        return json.load(f)


def save_reference_data(fname, recordings):
    base_dir = Path(__file__).resolve().parent
    fpath = base_dir.joinpath('reference', 'data', fname + '.json')
    with fpath.open("w") as f:
        json.dump(recordings, f)
        raise AssertionError("Regenerating reference data ... do not commit with this active in tests!")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2013-17 Richard Hull and contributors
# See LICENSE.rst for details.

"""
Tests for the :py:class:`luma.lcd.device.ht1621` device.
"""

from luma.lcd.device import ht1621
from luma.core.virtual import sevensegment
from unittest.mock import Mock


class MockHT1621:
    """
    Mock the HT1621 internal state machine.

    Decodes the 3-pin serial data back into
    the expected data packets.
    """

    HIGH = 1
    LOW = 0
    OUT = 1
    IN = 0

    def __init__(self, WR=11, DAT=10, CS=8):
        self._pin_wr = WR
        self._pin_dat = DAT
        self._pin_cs = CS
        self.reset_mock()

    def command(self, data):
        """
        Build a command representation.
        """
        return ('cmd', data)

    def data(self, data):
        """
        Built a data representation.
        """
        return ('data', data)

    def setup(self, pin, state):
        """
        Mock GPIO.setup.
        """
        self._pin_states[pin] = state

    def output(self, pin, state):
        """
        Mock GPIO.output.
        """
        if pin == self._pin_wr:
            if self._state_wr == 0 and state == 1:  # Rising edge
                self._data <<= 1
                self._data |= self._state_dat
                self._bit_count += 1
            self._state_wr = state

        if pin == self._pin_dat and self._state_wr == 0:
            self._state_dat = state

        # Handle 4-bit command SOF
        if self._bit_count == 4 and self._state == 'none' and self._data == 0b1000:
            self._state = 'cmd'
            self._data = 0
            self._bit_count = 0

        # Handle 3-bit data SOF
        if self._bit_count == 3 and self._state == 'none' and self._data == 0b101:
            self._state = 'addr'
            self._data = 0
            self._bit_count = 0

        # Discard 6-bit address (it's always 0)
        if self._bit_count == 6 and self._state == 'addr':
            self._state = 'data'
            self._data = 0
            self._bit_count = 0

        # If we hit 8 bits, treat it as one packet
        if self._bit_count == 8:
            self._values.append((self._state, self._data))
            self._data = 0
            self._bit_count = 0

        # Reset state machine on CS rising edge
        if pin == self._pin_cs:
            if self._state_cs == 0 and state == 1:  # Rising edge
                self._data = 0
                self._bit_count = 0
                self._state = 'none'

    def reset_mock(self):
        """
        Reset state machine.
        """
        self._state_wr = 0     # Read/Write State
        self._state_dat = 0    # Data state
        self._state_cs = 0     # Chip-select state
        self._data = 0         # Current value
        self._bit_count = 0    # Count of bits in _data
        self._values = []      # All clocked-in values
        self._pin_states = {}  # Pin IO states
        self._state = 'none'   # Current state

    def cleanup(self, *args):
        """
        Mock GPIO.cleanup.
        """
        self.reset_mock()

    def get_data(self):
        """
        Return stored values.
        """
        return self._values

    def get_pin_states(self):
        """
        Return stored pin output states.
        """
        return self._pin_states


gpio = MockHT1621()


def setup_function(function):
    """
    Called before a test runs.
    """
    gpio.reset_mock()


def test_init_6x8():
    ht1621(gpio, serial_interface=Mock())

    assert gpio.get_pin_states() == {
        11: gpio.OUT,
        10: gpio.OUT,
        8: gpio.OUT,
        18: gpio.OUT
    }

    assert gpio.get_data() == [
        gpio.command(0x30),  # Internal RC oscillator @ 256KHz
        gpio.command(0x52),  # 1/2 Bias and 4 commons
        gpio.command(0x02),  # System enable
        gpio.data(0),        # Column Data
        gpio.data(0),        # "
        gpio.data(0),        #
        gpio.data(0),        #
        gpio.data(0),        #
        gpio.data(0),        #
        gpio.command(0x06)   # Display On
    ]


def test_cleanup():
    device = ht1621(gpio)
    gpio.reset_mock()
    device.cleanup()
    assert gpio.get_data() == []  # No activity unless persist is True


def test_hide():
    device = ht1621(gpio)
    gpio.reset_mock()
    device.hide()
    assert gpio.get_data() == [gpio.command(0x04)]  # Display Off


def test_show():
    device = ht1621(gpio)
    gpio.reset_mock()
    device.show()
    assert gpio.get_data() == [gpio.command(0x06)]  # Display On


def test_display():
    device = ht1621(gpio)
    gpio.reset_mock()

    sevensegment(device).text = "HELLO"

    assert gpio.get_data() == [
        gpio.data(0), gpio.data(0), gpio.data(0),     # _ _ _
        gpio.data(0), gpio.data(0), gpio.data(0),     # _ _ _
        gpio.data(0), gpio.data(125), gpio.data(13),  # _ O L
        gpio.data(13), gpio.data(31), gpio.data(103)  # L E H
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2014-17 Richard Hull and contributors
# See LICENSE.rst for details.

"""
Tests for the :py:mod:`luma.lcd.segment_mapper` module.
"""

from luma.core.util import mutable_string
from luma.lcd.segment_mapper import dot_muncher


def test_dot_muncher_without_dots():
    buf = mutable_string("Hello world")
    results = dot_muncher(buf)
    assert list(results) == [0x67, 0x3f, 0x05, 0x05, 0x4e, 0x00, 0x08, 0x4e, 0x06, 0x05, 0x6e]


def test_dot_muncher_with_dot():
    buf = mutable_string("3.14159")
    results = dot_muncher(buf)
    assert list(results) == [0x7a, 0x60 | 0x80, 0x63, 0x60, 0x5b, 0x7b]


def test_dot_muncher_with_dot_at_end():
    buf = mutable_string("  525920")
    buf[7:] = ".0"
    print(buf)
    results = dot_muncher(buf)
    assert list(results) == [0x00, 0x00, 0x5b, 0x3e, 0x5b, 0x7b, 0x3e, 0x7d | 0x80]


def test_dot_muncher_with_multiple_dot():
    buf = mutable_string("127.0.0.1")
    results = dot_muncher(buf)
    assert list(results) == [0x60, 0x3e, 0x70, 0x7d | 0x80, 0x7d | 0x80, 0x60 | 0x80]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Richard Hull and contributors
# See LICENSE.rst for details.

"""
Tests for the :py:class:`luma.lcd.device.st7789` device.
"""
import pytest

from luma.lcd.device import st7789
from luma.core.render import canvas

from baseline_data import get_reference_data, primitives
from helpers import serial
from unittest.mock import Mock


def test_init_240x240():
    recordings = []

    def data(data):
        recordings.extend(data)

    def command(*cmd):
        recordings.extend(['command', list(cmd)[0], 'data', *list(cmd)[1:]])

    serial.command.side_effect = command
    serial.data.side_effect = data

    st7789(serial, gpio=Mock())

    assert serial.data.called
    assert serial.command.called

    assert recordings == [
        'command', 54, 'data', 112,
        'command', 58, 'data', 6,
        'command', 178, 'data', 12, 12, 0, 51, 51,
        'command', 183, 'data', 53,
        'command', 187, 'data', 25,
        'command', 192, 'data', 44,
        'command', 194, 'data', 1,
        'command', 195, 'data', 18,
        'command', 196, 'data', 32,
        'command', 198, 'data', 15,
        'command', 208, 'data', 164, 161,
        'command', 224, 'data', 208, 4, 13, 17, 19, 43, 63, 84, 76, 24, 13, 11, 31, 35,
        'command', 225, 'data', 208, 4, 12, 17, 19, 44, 63, 68, 81, 47, 31, 31, 32, 35,
        'command', 33, 'data',
        'command', 17, 'data',
        'command', 41, 'data',
        'command', 42, 'data', 0, 0, 0, 239,
        'command', 43, 'data', 0, 0, 0, 239,
        'command', 44, 'data', *([0] * (240 * 240 * 3)),
        'command', 41, 'data'
    ]


def test_contrast():
    device = st7789(serial, gpio=Mock())
    serial.reset_mock()
    with pytest.raises(AssertionError):
        device.contrast(300)


def test_hide():
    device = st7789(serial, gpio=Mock())
    serial.reset_mock()
    device.hide()
    serial.command.assert_called_once_with(40)


def test_show():
    device = st7789(serial, gpio=Mock())
    serial.reset_mock()
    device.show()
    serial.command.assert_called_once_with(41)


def test_display():
    device = st7789(serial, gpio=Mock())
    serial.reset_mock()

    recordings = []

    def data(data):
        recordings.extend(data)

    def command(*cmd):
        recordings.extend(['command', list(cmd)[0], 'data', *list(cmd)[1:]])

    serial.command.side_effect = command
    serial.data.side_effect = data

    # Use the same drawing primitives as the demo
    with canvas(device) as draw:
        primitives(device, draw)

    assert serial.data.called
    assert serial.command.called

    # To regenerate test data, uncomment the following (remember not to commit though)
    # ================================================================================
    # from baseline_data import save_reference_data
    # save_reference_data("demo_st7789", recordings)

    assert recordings == get_reference_data('demo_st7789')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2013-17 Richard Hull and contributors
# See LICENSE.rst for details.

"""
Tests for the :py:class:`luma.lcd.device.pcd8544` device.
"""

from luma.lcd.device import pcd8544
from luma.core.render import canvas

from baseline_data import get_reference_data, primitives
from helpers import serial, setup_function  # noqa: F401
from unittest.mock import Mock, call


def test_init_84x48():
    pcd8544(serial, gpio=Mock())
    serial.command.assert_has_calls([
        call(33, 20, 176, 32),
        call(32, 128, 64),
        call(12)
    ])

    # Next 1024 are all data: zero's to clear the RAM
    # (1024 = 128 * 64 / 8)
    serial.data.assert_called_once_with([0] * (84 * 48 // 8))


def test_hide():
    device = pcd8544(serial, gpio=Mock())
    serial.reset_mock()
    device.hide()
    serial.command.assert_called_once_with(8)


def test_show():
    device = pcd8544(serial, gpio=Mock())
    serial.reset_mock()
    device.show()
    serial.command.assert_called_once_with(12)


def test_display():
    device = pcd8544(serial, gpio=Mock())
    serial.reset_mock()

    # Use the same drawing primitives as the demo
    with canvas(device) as draw:
        primitives(device, draw)

    # Initial command to reset the display
    serial.command.assert_called_once_with(32, 128, 64)

    # To regenerate test data, uncomment the following (remember not to commit though)
    # ================================================================================
    # from baseline_data import save_reference_data
    # save_reference_data("demo_pcd8544", serial.data.call_args.args[0])

    # Next 1024 bytes are data representing the drawn image
    serial.data.assert_called_once_with(get_reference_data('demo_pcd8544'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2013-17 Richard Hull and contributors
# See LICENSE.rst for details.

"""
Tests for the :py:class:`luma.lcd.device.st7735` device.
"""

import pytest

from luma.lcd.device import st7735
from luma.core.render import canvas
from luma.core.framebuffer import full_frame

from baseline_data import get_reference_data, primitives
from helpers import serial, setup_function, assert_invalid_dimensions  # noqa: F401
from unittest.mock import Mock


def test_init_160x128():
    recordings = []

    def data(data):
        recordings.append({'data': data})

    def command(*cmd):
        recordings.append({'command': list(cmd)})

    serial.command.side_effect = command
    serial.data.side_effect = data

    st7735(serial, gpio=Mock(), framebuffer=full_frame())

    assert serial.data.called
    assert serial.command.called

    assert recordings == [
        {'command': [1]},
        {'command': [17]},
        {'command': [177]}, {'data': [1, 44, 45]},
        {'command': [178]}, {'data': [1, 44, 45]},
        {'command': [179]}, {'data': [1, 44, 45, 1, 44, 45]},
        {'command': [180]}, {'data': [7]},
        {'command': [192]}, {'data': [162, 2, 132]},
        {'command': [193]}, {'data': [197]},
        {'command': [194]}, {'data': [10, 0]},
        {'command': [195]}, {'data': [138, 42]},
        {'command': [196]}, {'data': [138, 238]},
        {'command': [197]}, {'data': [14]},
        {'command': [54]}, {'data': [96]},
        {'command': [32]},
        {'command': [58]}, {'data': [6]},
        {'command': [19]},
        {'command': [224]}, {'data': [15, 26, 15, 24, 47, 40, 32, 34, 31, 27, 35, 55, 0, 7, 2, 16]},
        {'command': [225]}, {'data': [15, 27, 15, 23, 51, 44, 41, 46, 48, 48, 57, 63, 0, 7, 3, 16]},
        {'command': [42]}, {'data': [0, 0, 0, 159]},
        {'command': [43]}, {'data': [0, 0, 0, 127]},
        {'command': [44]}, {'data': [0] * (160 * 128 * 3)},
        {'command': [41]}
    ]


def test_init_128x128():
    recordings = []

    def data(data):
        recordings.append({'data': data})

    def command(*cmd):
        recordings.append({'command': list(cmd)})

    serial.command.side_effect = command
    serial.data.side_effect = data

    st7735(serial, gpio=Mock(), width=128, height=128, framebuffer=full_frame())

    assert serial.data.called
    assert serial.command.called

    assert recordings == [
        {'command': [1]},
        {'command': [17]},
        {'command': [177]}, {'data': [1, 44, 45]},
        {'command': [178]}, {'data': [1, 44, 45]},
        {'command': [179]}, {'data': [1, 44, 45, 1, 44, 45]},
        {'command': [180]}, {'data': [7]},
        {'command': [192]}, {'data': [162, 2, 132]},
        {'command': [193]}, {'data': [197]},
        {'command': [194]}, {'data': [10, 0]},
        {'command': [195]}, {'data': [138, 42]},
        {'command': [196]}, {'data': [138, 238]},
        {'command': [197]}, {'data': [14]},
        {'command': [54]}, {'data': [96]},
        {'command': [32]},
        {'command': [58]}, {'data': [6]},
        {'command': [19]},
        {'command': [224]}, {'data': [15, 26, 15, 24, 47, 40, 32, 34, 31, 27, 35, 55, 0, 7, 2, 16]},
        {'command': [225]}, {'data': [15, 27, 15, 23, 51, 44, 41, 46, 48, 48, 57, 63, 0, 7, 3, 16]},
        {'command': [42]}, {'data': [0, 0, 0, 127]},
        {'command': [43]}, {'data': [0, 0, 0, 127]},
        {'command': [44]}, {'data': [0] * (128 * 128 * 3)},
        {'command': [41]}
    ]


def test_init_160x80():
    recordings = []

    def data(data):
        recordings.append({'data': data})

    def command(*cmd):
        recordings.append({'command': list(cmd)})

    serial.command.side_effect = command
    serial.data.side_effect = data

    st7735(serial, gpio=Mock(), width=160, height=80, framebuffer=full_frame())

    assert serial.data.called
    assert serial.command.called

    assert recordings == [
        {'command': [1]},
        {'command': [17]},
        {'command': [177]}, {'data': [1, 44, 45]},
        {'command': [178]}, {'data': [1, 44, 45]},
        {'command': [179]}, {'data': [1, 44, 45, 1, 44, 45]},
        {'command': [180]}, {'data': [7]},
        {'command': [192]}, {'data': [162, 2, 132]},
        {'command': [193]}, {'data': [197]},
        {'command': [194]}, {'data': [10, 0]},
        {'command': [195]}, {'data': [138, 42]},
        {'command': [196]}, {'data': [138, 238]},
        {'command': [197]}, {'data': [14]},
        {'command': [54]}, {'data': [96]},
        {'command': [32]},
        {'command': [58]}, {'data': [6]},
        {'command': [19]},
        {'command': [224]}, {'data': [15, 26, 15, 24, 47, 40, 32, 34, 31, 27, 35, 55, 0, 7, 2, 16]},
        {'command': [225]}, {'data': [15, 27, 15, 23, 51, 44, 41, 46, 48, 48, 57, 63, 0, 7, 3, 16]},
        {'command': [42]}, {'data': [0, 0, 0, 159]},
        {'command': [43]}, {'data': [0, 0, 0, 79]},
        {'command': [44]}, {'data': [0] * (160 * 80 * 3)},
        {'command': [41]}
    ]


def test_init_invalid_dimensions():
    """
    ST7735 LCD with an invalid resolution raises a
    :py:class:`luma.core.error.DeviceDisplayModeError`.
    """
    assert_invalid_dimensions(st7735, serial, 128, 77)


def test_offsets():
    recordings = []

    def data(data):
        recordings.append({'data': data})

    def command(*cmd):
        recordings.append({'command': list(cmd)})

    serial.command.side_effect = command
    serial.data.side_effect = data

    st7735(serial, gpio=Mock(), width=128, height=128, h_offset=2, v_offset=1, framebuffer=full_frame())

    assert serial.data.called
    assert serial.command.called

    assert recordings == [
        {'command': [1]},
        {'command': [17]},
        {'command': [177]}, {'data': [1, 44, 45]},
        {'command': [178]}, {'data': [1, 44, 45]},
        {'command': [179]}, {'data': [1, 44, 45, 1, 44, 45]},
        {'command': [180]}, {'data': [7]},
        {'command': [192]}, {'data': [162, 2, 132]},
        {'command': [193]}, {'data': [197]},
        {'command': [194]}, {'data': [10, 0]},
        {'command': [195]}, {'data': [138, 42]},
        {'command': [196]}, {'data': [138, 238]},
        {'command': [197]}, {'data': [14]},
        {'command': [54]}, {'data': [96]},
        {'command': [32]},
        {'command': [58]}, {'data': [6]},
        {'command': [19]},
        {'command': [224]}, {'data': [15, 26, 15, 24, 47, 40, 32, 34, 31, 27, 35, 55, 0, 7, 2, 16]},
        {'command': [225]}, {'data': [15, 27, 15, 23, 51, 44, 41, 46, 48, 48, 57, 63, 0, 7, 3, 16]},
        {'command': [42]}, {'data': [0, 2, 0, 129]},
        {'command': [43]}, {'data': [0, 1, 0, 128]},
        {'command': [44]}, {'data': [0] * (128 * 128 * 3)},
        {'command': [41]}
    ]


def test_contrast():
    device = st7735(serial, gpio=Mock())
    serial.reset_mock()
    with pytest.raises(AssertionError):
        device.contrast(300)


def test_hide():
    device = st7735(serial, gpio=Mock())
    serial.reset_mock()
    device.hide()
    serial.command.assert_called_once_with(40)


def test_show():
    device = st7735(serial, gpio=Mock())
    serial.reset_mock()
    device.show()
    serial.command.assert_called_once_with(41)


def test_display():
    device = st7735(serial, gpio=Mock(), framebuffer=full_frame())
    serial.reset_mock()

    recordings = []

    def data(data):
        recordings.append({'data': data})

    def command(*cmd):
        recordings.append({'command': list(cmd)})

    serial.command.side_effect = command
    serial.data.side_effect = data

    # Use the same drawing primitives as the demo
    with canvas(device) as draw:
        primitives(device, draw)

    assert serial.data.called
    assert serial.command.called

    # To regenerate test data, uncomment the following (remember not to commit though)
    # ================================================================================
    # from baseline_data import save_reference_data
    # save_reference_data("demo_st7735", recordings)

    assert recordings == get_reference_data('demo_st7735')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2017 Richard Hull and contributors
# See LICENSE.rst for details.

"""
Tests for the :py:class:`luma.lcd.device.uc1701x` device.
"""

from luma.lcd.device import uc1701x
from luma.core.render import canvas

from baseline_data import get_reference_data, primitives
from helpers import serial
from unittest.mock import Mock, call


def test_init_128x64():
    """
    UC1701X LCD with a 128 x 64 resolution works correctly.
    """
    uc1701x(serial, gpio=Mock())
    serial.command.assert_has_calls([
        # Initial burst are initialization commands
        call(226),
        call(44),
        call(46),
        call(47),
        call(248, 0),
        call(35),
        call(162),
        call(192),
        call(161),
        call(172),
        call(166),
        call(165),
        call(164),
        # set contrast
        call(129, 44),
        # reset the display
        call(176, 4, 16),
        call(177, 4, 16),
        call(178, 4, 16),
        call(179, 4, 16),
        call(180, 4, 16),
        call(181, 4, 16),
        call(182, 4, 16),
        call(183, 4, 16),
        call(175)])

    # Next 1024 are all data: zero's to clear the RAM
    # (1024 = 128 * 64 / 8)
    serial.data.assert_has_calls([call([0] * 128)] * 8)


def test_display():
    """
    UC1701X LCD screen can draw and display an image.
    """
    device = uc1701x(serial, gpio=Mock())
    serial.reset_mock()

    recordings = []

    def data(data):
        recordings.append({'data': data})

    def command(*cmd):
        recordings.append({'command': list(cmd)})

    serial.command = Mock(side_effect=command, unsafe=True)
    serial.data = Mock(side_effect=data, unsafe=True)

    # Use the same drawing primitives as the demo
    with canvas(device) as draw:
        primitives(device, draw)

    serial.data.assert_called()
    serial.command.assert_called()

    # To regenerate test data, uncomment the following (remember not to commit though)
    # ================================================================================
    # from baseline_data import save_reference_data
    # save_reference_data("demo_uc1701x", recordings)

    assert recordings == get_reference_data('demo_uc1701x')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2013-2023 Richard Hull and contributors
# See LICENSE.rst for details.

# luma.lcd documentation build configuration file, created by
# sphinx-quickstart on Wed Oct  5 00:14:54 2016.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import os
import sys
from datetime import datetime

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.

sys.path.insert(0, os.path.abspath('..'))

from luma.lcd import __version__ as version
from luma.lcd.device import __all__ as supported_devices

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
needs_sphinx = '1.5.3'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.intersphinx',
    'sphinx.ext.autosectionlabel',
    'sphinx.ext.todo',
    'sphinx.ext.coverage',
    'sphinx.ext.ifconfig',
    'sphinx.ext.viewcode',
    'sphinx.ext.inheritance_diagram',
    'sphinx.ext.extlinks'
]

# Add prefix each section label to prevent duplicates
autosectionlabel_prefix_document = True

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'Luma.LCD: Display drivers for {0}'.format(
    ', '.join(supported_devices).upper())
author = 'Richard Hull and contributors'
copyright = u'2013-{0}, {1}'.format(datetime.now().year, author)

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.

# The full version, including alpha/beta/rc tags.
release = version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False

# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

# Output file base name for HTML help builder.
htmlhelp_basename = 'luma.lcd_doc'

# -- Options for LaTeX output ---------------------------------------------

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'luma.lcd.tex', 'Luma.LCD Documentation',
     author, 'manual'),
]

# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'luma.lcd', 'luma.LCD Documentation',
     [author], 1)
]

# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'luma.lcd', 'Luma.LCD Documentation',
     author, 'luma.lcd', project,
     'Miscellaneous'),
]

# Configuration for intersphinx.
intersphinx_mapping = {
    'python': ('https://docs.python.org/3', None),
    'pillow': ('https://pillow.readthedocs.io/en/latest', None),
    'luma.core': ('https://luma-core.readthedocs.io/en/latest', None),
    'luma.emulator': ('https://luma-emulator.readthedocs.io/en/latest', None)
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2020-2022 Richard Hull and contributors
# See LICENSE.rst for details.

"""
Tests for the :py:class:`luma.lcd.device.hd44780` device.
"""

from luma.lcd.device import hd44780
from luma.core.render import canvas
from luma.core.util import bytes_to_nibbles
from luma.core.framebuffer import full_frame, diff_to_previous
from luma.lcd.const import hd44780 as CONST
from luma.lcd.big_digits import device as big_digits
from pathlib import Path

from PIL import Image, ImageDraw
from unittest.mock import Mock, call

interface = Mock(unsafe=True, _bitmode=4)
gpio = Mock()


def test_init_4bitmode():
    """
    Test initialization of display using 4 bit mode
    """
    hd44780(interface, gpio=gpio, framebuffer=full_frame())

    to_8 = \
        [call(0x3), call(0x3), call(0x3, 0x3)] * 3
    to_4 = \
        [call(0x3), call(0x3), call(0x3, 0x02)]

    fs = [CONST.FUNCTIONSET | CONST.DL4 | CONST.LINES2]

    calls = \
        to_8 + \
        to_4 + \
        [call(*bytes_to_nibbles(fs))] + \
        [call(*bytes_to_nibbles([CONST.DISPLAYOFF]))] + \
        [call(*bytes_to_nibbles([CONST.CLEAR]))] + \
        [call(*bytes_to_nibbles([CONST.ENTRY]))] + \
        [call(*bytes_to_nibbles([CONST.DISPLAYON]))] + \
        [call(*bytes_to_nibbles([CONST.DDRAMADDR]))] + \
        [call(*bytes_to_nibbles([CONST.DDRAMADDR | CONST.LINES[1]]))] + \
        [call(*bytes_to_nibbles([CONST.CLEAR]))]

    interface.command.assert_has_calls(calls)

    # Data to clear the screen
    calls = \
        [call(bytes_to_nibbles([0x20] * 16))] + \
        [call(bytes_to_nibbles([0x20] * 16))]

    interface.data.assert_has_calls(calls)


def test_init_8bitmode():
    """
    Test initialization of display using 4 bit mode
    """
    interface._bitmode = 8
    hd44780(interface, gpio=gpio, framebuffer=full_frame())

    to_8 = \
        [call(0x30)] * 3

    fs = [CONST.FUNCTIONSET | CONST.DL8 | CONST.LINES2]

    calls = \
        to_8 + \
        [call(*fs)] + \
        [call(*[CONST.DISPLAYOFF])] + \
        [call(*[CONST.CLEAR])] + \
        [call(*[CONST.ENTRY])] + \
        [call(*[CONST.DISPLAYON])] + \
        [call(*[CONST.DDRAMADDR])] + \
        [call(*[CONST.DDRAMADDR | CONST.LINES[1]])] + \
        [call(*[CONST.CLEAR])]

    interface.command.assert_has_calls(calls)

    # Data to clear the screen
    calls = \
        [call([0x20] * 16)] + \
        [call([0x20] * 16)]

    interface.data.assert_has_calls(calls)


def test_display():
    """
    Test the display with a line of text and a rectangle to demonstrate correct
    functioning of the auto-create feature
    """
    device = hd44780(interface, bitmode=8, gpio=gpio, framebuffer=full_frame())
    interface.reset_mock()

    # Use canvas to create a screen worth of data
    with canvas(device) as drw:
        # Include unprintable character to show it gets ignored
        width = device.font.getlength('This is a test\uFFFF')
        drw.text(((80 - width) // 2, 0), 'This is a test\uFFFF', font=device.font, fill='white')
        drw.rectangle((10, 10, 69, 14), fill='black', outline='white')
        drw.rectangle((10, 10, 49, 14), fill='white', outline='white')

    # Send DDRAMADDR and ascii for the line of text
    line1 = [call.command(0x80)] + \
        [call.data([0x20, 0x54, 0x68, 0x69, 0x73, 0x20, 0x69, 0x73, 0x20, 0x61, 0x20, 0x74, 0x65, 0x73, 0x74, 0x20])]

    # Create custom characters for the scrollbar
    custom = [call.command(0x40), call.data([0x00, 0x00, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x00])] + \
        [call.command(0x48), call.data([0x00, 0x00, 0x1f, 0x00, 0x00, 0x00, 0x1f, 0x00])] + \
        [call.command(0x50), call.data([0x00, 0x00, 0x1f, 0x01, 0x01, 0x01, 0x1f, 0x00])]

    # Print the resulting custom characters to form the image of the scrollbar
    line2 = [call.command(0xc0), call.data([0x20, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x02, 0x20, 0x20])]

    interface.assert_has_calls(line1 + custom + line2)


def test_custom_full():
    """
    Auto-create feature runs out of custom character space
    """
    device = hd44780(interface, undefined='_', bitmode=8, gpio=gpio, framebuffer=diff_to_previous(num_segments=1))

    # Consume 8 special character positions
    img = Image.new('1', (80, 16), 0)
    drw = ImageDraw.Draw(img)
    for i in range(8):
        drw.rectangle((i * 5, 0, (i + 1) * 5, i), fill='white', outline='white')
    device.display(img)

    interface.reset_mock()

    # Consume one more (on the last char position on screen)
    drw.line((75, 8, 79, 15), fill='white')
    device.display(img)

    # Existing custom chars should not be replaced and the last custom char should be undefined
    assert call.command(0x40) not in interface.mock_calls
    interface.assert_has_calls([call.command(0xcf), call.data([0x5f])])


def test_get_font():
    """
    Test get font capability by requesting two fonts and printing a single
    character from each that will be different between the two fonts
    """
    device = hd44780(interface, bitmode=8, gpio=gpio, framebuffer=full_frame())

    img = Image.new('1', (10, 8), 0)
    a00 = device.get_font(0)
    a02 = device.get_font(1)
    drw = ImageDraw.Draw(img)

    assert a00.getsize('\u00E0') == (5, 8)

    drw.text((0, 0), '\u00E0', font=a00, fill='white')
    drw.text((5, 0), '\u00E0', font=a02, fill='white')

    assert img.tobytes() == \
        b'\x02\x00\x01\x00H\x00\xab\x80\x90@\x93\xc0l@\x03\xc0'


def test_no_contrast():
    """
    HD44780 should ignore requests to change contrast
    """
    device = hd44780(interface, bitmode=8, gpio=gpio, framebuffer=full_frame())
    device.contrast(100)


def test_i2c_backlight():
    """
    Test of i2c_backlight
    """

    def _mask(pin):
        """
        Return a mask that contains a 1 in the pin position
        """
        return 1 << pin

    interface = Mock(unsafe=True, _bitmode=4, _backlight_enabled=0, _mask=_mask)
    hd44780(interface, bitmode=8, backpack_pin=3, gpio=gpio, framebuffer=full_frame())

    assert interface._backlight_enabled == 8


def test_i2c_does_not_support_backlight():
    """
    An exception is thrown if supplied interface does not support a backlight
    """
    import luma.core
    interface = Mock(spec_set=luma.core.interface.serial.i2c)
    flag = False
    try:
        hd44780(interface, gpio=gpio, backpack_pin=3, framebuffer=full_frame())
    except luma.core.error.UnsupportedPlatform as ex:
        assert str(ex) == "This I2C interface does not support a backlight"
        flag = True

    assert flag, "Expected exception but none occured"


def test_unsupported_display_mode():
    """
    An exception is thrown if an unsupported display mode is requested
    """
    import luma.core
    try:
        hd44780(interface, width=12, height=3, gpio=gpio, framebuffer=full_frame())
    except luma.core.error.DeviceDisplayModeError as ex:
        assert str(ex) == "Unsupported display mode: 12 x 3"


def test_custom_backlight():
    """
    The custom backlight should be enabled after init.
    """
    backlight = Mock()
    hd44780(interface, backlight=backlight)
    backlight.assert_called_once_with(True)


def test_image_move_by_pixel():
    """
    A static image is moved pixel-by-pixel.
    New custom characters should be created on each loop.
    """
    img = Image.open(str(Path(__file__).resolve().parent.joinpath('reference', 'images', 'custom6.png')))
    device = hd44780(interface, gpio=gpio, bitmode=8, undefined='_')
    interface.reset_mock()

    calls = [call.command(CONST.CGRAMADDR + i * 8) for i in range(6)]
    undefined = [call(0x5f)]  # Undefined character '_'

    for i in range(5):
        with canvas(device) as draw:
            interface.reset_mock()
            draw.bitmap((i, 0), img, fill='white')
        interface.assert_has_calls(calls, any_order=True)
        assert undefined not in interface.data.mock_calls


def test_image_move_by_segment():
    """
    A static image is moved segment-by-segment.
    Custom characters should be created the first time only, then re-used from memory.
    """
    img = Image.open(str(Path(__file__).resolve().parent.joinpath('reference', 'images', 'custom6.png')))
    device = hd44780(interface, gpio=gpio, bitmode=8, undefined='_')
    interface.reset_mock()

    calls = [call.command(CONST.CGRAMADDR + i * 8) for i in range(6)]
    undefined = [call(0x5f)]  # Undefined character '_'

    for i in range(5):
        with canvas(device) as draw:
            interface.reset_mock()
            draw.bitmap((i * 5, 0), img, fill='white')
        for c in calls:
            if i == 0:
                assert c in interface.mock_calls
            else:
                assert c not in interface.mock_calls
        assert undefined not in interface.data.mock_calls


def test_big_digits():
    """
    Test that custom characters are constructed to form the number 5 from font 'classic'.
    """
    interface._bitmode = 8
    device = hd44780(serial_interface=interface, gpio=gpio, bitmode=8)
    interface.reset_mock()

    big_digit_device = big_digits(device=device, selected_font='classic')
    big_digit_device.text = '5'

    interface.assert_has_calls([call.command(CONST.CGRAMADDR), call.data([0x1f, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x1f]),
                               call.command(CONST.CGRAMADDR + 8), call.data([0x1f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f]),
                               call.command(CONST.CGRAMADDR + 16), call.data([0x1f, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x1f])], any_order=True)



# -*- coding: utf-8 -*-
# Copyright (c) 2017-2022 Richard Hull and contributors
# See LICENSE.rst for details.

import pytest
from unittest.mock import Mock

import luma.core.error

serial = Mock(unsafe=True)


def setup_function(function):
    """
    Called before a test runs.
    """
    serial.reset_mock()
    serial.command.side_effect = None


def assert_invalid_dimensions(deviceType, serial_interface, width, height):
    """
    Assert an invalid resolution raises a
    :py:class:`luma.core.error.DeviceDisplayModeError`.
    """
    with pytest.raises(luma.core.error.DeviceDisplayModeError) as ex:
        deviceType(serial_interface, gpio=Mock(), width=width, height=height)
    assert f"Unsupported display mode: {width} x {height}" in str(ex.value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2013-17 Richard Hull and contributors
# See LICENSE.rst for details.

import luma.core.error
from luma.lcd.device import backlit_device
from luma.core.device import dummy
from luma.core.interface.serial import noop, pcf8574
from unittest.mock import Mock
import pytest


gpio = Mock(unsafe=True)


class dummy_backlit_device(dummy, backlit_device):
    pass


def setup_function(function):
    gpio.reset_mock()
    gpio.BCM = 1
    gpio.OUT = 2
    gpio.HIGH = 3
    gpio.LOW = 4


def test_unsupported_platform():
    e = RuntimeError('Module not imported correctly!')
    errorgpio = Mock(unsafe=True)
    errorgpio.setup.side_effect = e

    try:
        backlit_device(serial_interface=noop(), gpio_LIGHT=19, gpio=errorgpio)
    except luma.core.error.UnsupportedPlatform as ex:
        assert str(ex) == 'GPIO access not available'
    else:
        pytest.fail("Didn't raise exception")


def test_init():
    gpio_LIGHT = 11
    backlit_device(serial_interface=noop(), gpio=gpio, gpio_LIGHT=gpio_LIGHT)
    gpio.setup.assert_called_once_with(gpio_LIGHT, gpio.OUT)
    gpio.output.assert_called_once_with(gpio_LIGHT, gpio.HIGH)


def test_gpio_cleanup():
    """
    Cleanup of backlit_device using a GPIO interface should turn light off and then
    clean up the GPIO pin associated with the backlight
    """
    gpio_LIGHT = 11
    device = dummy_backlit_device(gpio=gpio, gpio_LIGHT=gpio_LIGHT)
    gpio.reset_mock()
    device.cleanup()
    gpio.output.assert_called_once_with(gpio_LIGHT, gpio.LOW)
    gpio.cleanup.assert_called_once_with(gpio_LIGHT)


def test_pwm_cleanup():
    """
    Cleanup of backlit_device using a GPIO PWM interface should turn light off,
    shut off the PWM function and then clean up the GPIO pin associated with the
    backlight
    """
    gpio_LIGHT = 11
    device = dummy_backlit_device(gpio=gpio, gpio_LIGHT=gpio_LIGHT,
        pwm_frequency=362)
    gpio.reset_mock()
    device.cleanup()
    assert device.backlight._pwm.stop.called, \
        'PWM stop method should be called during cleanup'
    gpio.cleanup.assert_called_once_with(gpio_LIGHT)


def test_pcf8574_cleanup():
    """
    Cleanup of backlit device using a PCF8574 interface should not cleanup gpio
    """
    gpio_LIGHT = 11
    smbus = Mock(unsafe=True)
    serial = pcf8574(bus=smbus, address=0x27)
    gpio.reset_mock()
    device = dummy_backlit_device(serial_interface=serial, gpio=gpio, gpio_LIGHT=gpio_LIGHT)
    device.cleanup()
    assert not gpio.cleanup.called, 'GPIO cleanup not necessary with _backlight_enabled interface'


def test_active_low_enable_on():
    gpio_LIGHT = 14
    device = backlit_device(serial_interface=noop(), gpio=gpio, gpio_LIGHT=gpio_LIGHT, active_low=True)
    gpio.reset_mock()
    device.backlight(True)
    gpio.output.assert_called_once_with(gpio_LIGHT, gpio.LOW)


def test_active_low_enable_off():
    gpio_LIGHT = 19
    device = backlit_device(serial_interface=noop(), gpio=gpio, gpio_LIGHT=gpio_LIGHT, active_low=True)
    gpio.reset_mock()
    device.backlight(False)
    gpio.output.assert_called_once_with(gpio_LIGHT, gpio.HIGH)


def test_active_high_enable_on():
    gpio_LIGHT = 14
    device = backlit_device(serial_interface=noop(), gpio=gpio, gpio_LIGHT=gpio_LIGHT, active_low=False)
    gpio.reset_mock()
    device.backlight(True)
    gpio.output.assert_called_once_with(gpio_LIGHT, gpio.HIGH)


def test_active_high_enable_off():
    gpio_LIGHT = 19
    device = backlit_device(serial_interface=noop(), gpio=gpio, gpio_LIGHT=gpio_LIGHT, active_low=False)
    gpio.reset_mock()
    device.backlight(False)
    gpio.output.assert_called_once_with(gpio_LIGHT, gpio.LOW)


def test_pwm_turn_off():
    gpio_LIGHT = 18
    pwm_mock = Mock()
    gpio.PWM.return_value = pwm_mock
    device = backlit_device(serial_interface=noop(), gpio=gpio, gpio_LIGHT=gpio_LIGHT, pwm_frequency=100)
    gpio.PWM.assert_called_once_with(gpio_LIGHT, 100)
    gpio.reset_mock()
    device.backlight(False)
    pwm_mock.ChangeDutyCycle.assert_called_once_with(0.0)


def test_pwm_turn_on():
    gpio_LIGHT = 18
    pwm_mock = Mock()
    gpio.PWM.return_value = pwm_mock
    device = backlit_device(serial_interface=noop(), gpio=gpio, gpio_LIGHT=gpio_LIGHT, pwm_frequency=100)
    gpio.PWM.assert_called_once_with(gpio_LIGHT, 100)
    gpio.reset_mock()
    device.backlight(True)
    pwm_mock.ChangeDutyCycle.assert_called_once_with(100.0)


def test_pwm_turn_50_percent():
    gpio_LIGHT = 18
    pwm_mock = Mock()
    gpio.PWM.return_value = pwm_mock
    device = backlit_device(serial_interface=noop(), gpio=gpio, gpio_LIGHT=gpio_LIGHT, pwm_frequency=100)
    gpio.PWM.assert_called_once_with(gpio_LIGHT, 100)
    gpio.reset_mock()
    device.backlight(50.0)
    pwm_mock.ChangeDutyCycle.assert_called_once_with(50.0)


def test_pwm_unsupported_platform():
    gpio_LIGHT = 18
    e = RuntimeError('Module not imported correctly!')
    gpio.PWM.side_effect = e

    try:
        backlit_device(serial_interface=noop(), gpio=gpio, gpio_LIGHT=gpio_LIGHT, pwm_frequency=100)
    except luma.core.error.UnsupportedPlatform as ex:
        assert str(ex) == 'GPIO access not available'
    else:
        pytest.fail("Didn't raise exception")
